<?php

namespace Drupal\entity_reference_media\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceEntityFormatter;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'entity_reference_media_entity' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_reference_media_entity",
 *   label = @Translation("Rendered entity"),
 *   description = @Translation("Display the referenced entities rendered by
 *   entity_view(), with optional field overrides."), field_types = {
 *     "entity_reference_media"
 *   }
 * )
 */
class EntityReferenceMediaFormatter extends EntityReferenceEntityFormatter {

  /**
   * The EntityField Manager service.
   */
  protected $entityFieldManager;

  /**
   * The EntityTypeBundleInfo.
   */
  protected $entityTypeBundleInfo;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->entityFieldManager = $container->get('entity_field.manager');
    $instance->entityTypeBundleInfo = $container->get('entity_type.bundle.info');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'custom_caption' => '',
        'video_start' => '',
        'video_end' => '',
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);

    $target_entity_type = $this->getFieldSetting('target_type');

    $mediaBundles = $this->entityTypeBundleInfo->getBundleInfo($target_entity_type);
    $options = array_combine(array_keys($mediaBundles), array_column($mediaBundles, 'label'));

    $elements['media_bundles'] = [
      '#type' => 'select',
      '#options' => $options,
      '#title' => t('Select a bundle'),
    ];

    $fieldOptions = $this->entityFieldManager->getFieldMapByFieldType('string')[$target_entity_type];

    foreach ($mediaBundles as $key => $item['label']) {
      $fields[$key][''] = t('-- Select --');

      foreach ($fieldOptions as $field => $fieldOption) {
        if (!empty($fieldOption['bundles'][$key])) {
          $fields[$key][$field] = $field;
        }
      }

      $elements['custom_caption'][$key] = [
        '#type' => 'select',
        '#options' => $fields[$key],
        '#title' => t('Custom Caption'),
        '#description' => t('Select the field you want to override with <em>custom_caption</em>'),
        '#default_value' => $this->getSetting('custom_caption')[$key],
        '#access' => !empty($this->fieldDefinition->getSetting('field_caption')[$key]),
        '#states' => [
          'visible' => [
            ':input[name*="media_bundles"]' => ['value' => $key],
          ],
        ],
      ];

      $elements['video_start'][$key] = [
        '#type' => 'select',
        '#options' => $fields[$key],
        '#title' => t('Video Start'),
        '#description' => t('Select the field you want to override with <em>video_start</em>'),
        '#default_value' => $this->getSetting('video_start')[$key],
        '#access' => !empty($this->fieldDefinition->getSetting('field_start_end')[$key]),
        '#states' => [
          'visible' => [
            ':input[name*="media_bundles"]' => ['value' => $key],
          ],
        ],
      ];

      $elements['video_end'][$key] = [
        '#type' => 'select',
        '#options' => $fields[$key],
        '#title' => t('Video End'),
        '#description' => t('Select the field you want to override with <em>video_end</em>'),
        '#default_value' => $this->getSetting('video_end')[$key],
        '#access' => !empty($this->fieldDefinition->getSetting('field_start_end')[$key]),
        '#states' => [
          'visible' => [
            ':input[name*="media_bundles"]' => ['value' => $key],
          ],
        ],
      ];
    }

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

//    $summary[] = $this->showCaptionField() ? t('<em>custom_caption</em> overrides <em>@override</em>', ['@override' => $this->getSetting('custom_caption')]) : [];
//    $summary[] = $this->showStartEndFields() ? t('<em>video_start</em> overrides <em>@override</em>', ['@override' => $this->getSetting('video_start')]) : [];
//    $summary[] = $this->showStartEndFields() ? t('<em>video_end</em> overrides <em>@override</em>', ['@override' => $this->getSetting('video_end')]) : [];
//    $summary = array_filter($summary);

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $view_mode = $this->getSetting('view_mode');
    $elements = [];

    foreach ($this->getEntitiesToView($items, $langcode) as $delta => $entity) {

      if (!$items->get($delta)->get('default_caption')->getCastedValue()) {
        // custom caption
        $customCaption = $this->getSetting('custom_caption')[$entity->bundle()];
        if ($entity->hasField($customCaption)) {
          $entity->set($customCaption, $items->get($delta)
            ->get('custom_caption')
            ->getValue());
        }
      }

      if (!$items->get($delta)->get('default_start_end')->getCastedValue()) {
        // video start
        $videoStart = $this->getSetting('video_start')[$entity->bundle()];
        if ($entity->hasField($videoStart)) {
          $entity->set($videoStart, $items->get($delta)
            ->get('video_start')
            ->getValue());
        }
        // video end
        $videoEnd = $this->getSetting('video_end')[$entity->bundle()];
        if ($entity->hasField($videoEnd)) {
          $entity->set($videoEnd, $items->get($delta)
            ->get('video_end')
            ->getValue());
        }
      }

      $view_builder = $this->entityTypeManager->getViewBuilder($entity->getEntityTypeId());
      $elements[$delta] = $view_builder->view($entity, $view_mode, $entity->language()
        ->getId());
    }

    return $elements;
  }

}
